@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: miscutils.cc 222 2005-07-31 13:11:05Z murrayc $ */
+/* $Id: miscutils.cc,v 1.6 2006/09/10 14:38:53 jjongsma Exp $ */
 
 /* Copyright (C) 2002 The gtkmm Development Team
  *
@@ -115,6 +115,21 @@ std::string get_current_dir()
   return std::string(buf.get());
 }
 
+std::string get_user_data_dir()
+{
+  return std::string(g_get_user_data_dir());
+}
+
+std::string get_user_config_dir()
+{
+  return std::string(g_get_user_config_dir());
+}
+
+std::string get_user_cache_dir()
+{
+  return std::string(g_get_user_cache_dir());
+}
+
 bool path_is_absolute(const std::string& filename)
 {
   return g_path_is_absolute(filename.c_str());
